#!/usr/bin/python
# -*- coding: utf-8 -*-
#This file is part of pydsl.
#
#pydsl is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.
#
#pydsl is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with pydsl.  If not, see <http://www.gnu.org/licenses/>.


__author__ = "Nestor Arocha"
__copyright__ = "Copyright 2008-2013, Nestor Arocha"
__email__ = "nesaro@gmail.com"

import logging
LOG = logging.getLogger(__name__)

class Memory(object):
    """Memory Abstraction"""
    def __init__(self):
        pass

    def load(self, index, **kwargs):
        raise NotImplementedError
    
    def save(self, element, identifier):
        raise NotImplementedError

    def __iter__(self):
        """Must use summary abstraction"""
        raise NotImplementedError
    
    def next(self):
        raise NotImplementedError

    def __next__(self):
        return self.next()

    def __getitem__(self, index):
        return self.load(index)

    def __setitem__(self, index, value):
        return self.save(value, index)

    def indexer(self):
        from pydsl.Memory.Search.Indexer import Indexer
        return Indexer(self)
    
    def searcher(self):
        from pydsl.Memory.Search.Searcher import MemorySearcher
        return MemorySearcher(self.indexer())
    
    def search(self, query):
        return self.searcher().search(query)

    def provided_iclasses(self):
        raise NotImplementedError

class LocalMemory(Memory):
    """Execution time memory"""
    def __init__(self):
        Memory.__init__(self)
        self.content = {}
    
    def load(self, index):
        return self.content[index]
    
    def save(self, element, identifier):
        self.content[str(identifier)] = element
    
    def __delitem__(self, key):
        del self.content[key]

    def __contains__(self, index):
        return index in self.content
    
    def __iter__(self):
        self.cacheindex = 0
        self.cache = []
        for element in self.content.values():
            self.cache.append(element.summary)
        return self
    
    def next(self):
        try:
            result = self.cache[self.cacheindex]
        except IndexError:
            raise StopIteration
        self.cacheindex += 1
        return result



#!/usr/bin/python
# -*- coding: utf-8 -*-
#This file is part of pydsl.
#
#pydsl is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.
#
#pydsl is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with pydsl.  If not, see <http://www.gnu.org/licenses/>.


from pydsl.Query import QueryEquality, QueryInclusion, QueryPartial
import collections
from pydsl.Abstract import ImmutableDict
import re

class Indexer(object):
    """Indexes memory content. Current implementation just copy Memory iterator (Immutabledicts) into a cache dict.
    This is possible because Memory iterator format is suitable for search, but it might change in the future"""
    def __init__(self, memory):
        self.memory = memory #One memory per indexer
        self.index = []
        self.__init_index()

    def __init_index(self):
        """Stores all data in the index"""
        for element in self.memory:
            if not isinstance(element, collections.Hashable):
                raise TypeError("Adding non hashable element %s to index" % str(element))
            self.index.append(element)

    def show_all(self):# -> list:
        """All elements inside index"""
        return list(self.index)

    def __update_index(self):
        """Updates index content"""
        for element in self.memory:
            if element not in self.index:
                self.index.append(element)

    @staticmethod
    def __get_left_value(left, element):
        if not '.' in left:
            return element[left]
        getlist = left.split('.')
        leftvalue = element[getlist[0]]
        for getindex in getlist[1:]:
            leftvalue = leftvalue.__getitem__(getindex)
        return leftvalue

    @staticmethod
    def __to_immutable(element):
        if isinstance(element, dict):
            return ImmutableDict(element)
        return element

    def search_index(self, queryterm): #-> set:
        result = set()
        right = queryterm.right
        left = queryterm.left
        for element in self.index:
            if not element:
                continue
            if isinstance(queryterm, QueryEquality):
                leftvalue = self.__get_left_value(left, element)
                if len(right)>2 and right[-1] == "/" and right[0] == "/":
                    #Regular Expression
                    rexp = re.compile(right[1:-1])
                    if rexp.match(leftvalue) is None:
                        continue
                elif right != leftvalue:
                    continue
            elif isinstance(queryterm, QueryInclusion):
                #a in b
                try:
                    list_to_search = element[left]
                    if isinstance(list_to_search, dict) and queryterm.dict_member == "values":
                        list_to_search = list(list_to_search.values())
                    elif isinstance(list_to_search, dict) and queryterm.dict_member == "keys":
                        list_to_search = list(list_to_search.keys())
                    if right[-1] == "/" and right[0] == "/":
                        #RegExp
                        rexp = re.compile(right[1:-1])
                        for item in list_to_search:
                            if rexp.match(item) is not None:
                                break
                        else:
                            continue
                    else:
                        #string
                        if right not in list_to_search:
                            continue
                except KeyError:
                    continue
            elif isinstance(queryterm, QueryPartial):
                if left not in element:
                    continue
                ismatch = True
                for key, value in right.items():
                    try:
                        if key not in element[left]:
                            ismatch = False
                            break
                        if element[left][key] != value:
                            ismatch = False
                    except KeyError:
                        continue
                if not ismatch:
                    continue
            result.add(self.__to_immutable(element))
        return result





#!/usr/bin/python3
# -*- coding: utf-8 -*-
#This file is part of pydsl.
#
#pydsl is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.
#
#pydsl is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with pydsl.  If not, see <http://www.gnu.org/licenses/>.

"""
guess to which type can belong an input. It works like the unix file command
"""

__author__ = "Nestor Arocha"
__copyright__ = "Copyright 2008-2013, Nestor Arocha"
__email__ = "nesaro@gmail.com"


import logging


if __name__ == "__main__":
    import argparse
    USAGE = "usage: %(prog)s [options]"
    PARSER = argparse.ArgumentParser(usage = USAGE)
    PARSER.add_argument("-d", "--debuglevel", action="store", type=int, dest="debuglevel", help="Sets debug level")
    PARSER.add_argument("-i", "--inputfile", action="store", dest="inputfile", help="input filename dict")
    PARSER.add_argument("-e", "--expression", action="store", dest="expression", help="input expression")
    ARGS = PARSER.parse_args()
    import sys
    if not ARGS.inputfile and not ARGS.expression:
        PARSER.error("options -i, -u or -e are required")
    if ARGS.inputfile and ARGS.expression:
        PARSER.error("options -i and -e can't be together")
    from pydsl.Config import load_default_memory
    load_default_memory()
    DEBUGLEVEL = 39
    if ARGS.debuglevel:
        DEBUGLEVEL = ARGS.debuglevel
    logging.basicConfig(level = DEBUGLEVEL)    
    from pydsl.Guess import Guesser
    guess = Guesser()
    if ARGS.inputfile:
        with open(ARGS.inputfile, "rb") as f:
            inputstr = f.read() 
        result = guess(inputstr)
    elif ARGS.expression:
        result = guess(ARGS.expression)
    else:
        print(USAGE)
        sys.exit(0)
    print(result)
    sys.exit(0)



#!/usr/bin/python
# -*- coding: utf-8 -*-
#This file is part of pydsl.
#
#pydsl is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.
#
#pydsl is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with pydsl.  If not, see <http://www.gnu.org/licenses/>.

__author__ = "Nestor Arocha"
__copyright__ = "Copyright 2008-2013, Nestor Arocha"
__email__ = "nesaro@gmail.com"

from pydsl.Grammar.Definition import GrammarDefinition, StringGrammarDefinition

class Token(object):
    """ Stores a token and the associated grammar definition.
    Alias is an alternative name for this token"""
    def __init__(self, string, gd=None, alias = None):
        if gd is None:
            gd = StringGrammarDefinition(string)
        if not isinstance(gd, GrammarDefinition):
            raise TypeError
        self.gd = gd
        self.string = string
        self.alias = alias

    def __str__(self):
        return self.string

    def __eq__(self, other):
        try:
            return self.gd == other.gd and self.string == other.string and self.alias == other.alias
        except AttributeError:
            return False



#!/usr/bin/python
# -*- coding: utf-8 -*-
#This file is part of pydsl.
#
#pydsl is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.
#
#pydsl is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with pydsl.  If not, see <http://www.gnu.org/licenses/>.

"""ListLibrary"""
from pydsl.Abstract import ImmutableDict
import logging
LOG = logging.getLogger("Storage.List")
from pydsl.Memory.Memory import Memory
from pydsl.Alphabet.Definition import Encoding



__author__ = "Nestor Arocha"
__copyright__ = "Copyright 2008-2013, Nestor Arocha"
__email__ = "nesaro@gmail.com"


class ListStorage(Memory):
    """Stores element in a python file using a python list"""
    def __init__(self, fullpath):
        Memory.__init__(self)
        self._content = {}
        from pydsl.Memory.Search.Searcher import MemorySearcher
        from pydsl.Memory.File.Python import getFileTuple
        (_, _, fileBaseName, _) = getFileTuple(fullpath)
        import imp
        myobj = imp.load_source(fileBaseName, fullpath)
        for element in myobj.mylist:
            self._content[self._generatekey(element)] = element
        self._searcher = MemorySearcher(self)

    def __iter__(self):
        self.index = 0
        self.cache = []
        self.cache += self.generate_all_summaries()
        return self

    def next(self):
        try:
            result = self.cache[self.index]
        except IndexError:
            raise StopIteration
        self.index += 1
        return result

    def _generatekey(self, element):
        """Generates a identifier for the input element to use in this memory"""
        raise NotImplementedError

    def generate_all_summaries(self):
        """generates the full list of element summaries"""
        raise NotImplementedError
        
    def __contains__(self, index):
        return index in self._content


class EncodingStorage(ListStorage):
    def _generatekey(self, element):
        return element

    def load(self, index):
        return Encoding(index)

    def generate_all_summaries(self):
        return tuple([ImmutableDict({"identifier":x, "value":x, "iclass":"Encoding"}) for x in self._content])



#!/usr/bin/python
# -*- coding: utf-8 -*-
#This file is part of pydsl.
#
#pydsl is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.
#
#pydsl is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with pydsl.  If not, see <http://www.gnu.org/licenses/>.

"""Python Transformers"""

__author__ = "Nestor Arocha"
__copyright__ = "Copyright 2008-2013, Nestor Arocha"
__email__ = "nesaro@gmail.com"

import logging
LOG = logging.getLogger(__name__)

def _load_checker(originaldic):
    """Converts {"channelname","type"} into {"channelname",instance}"""
    from pydsl.Memory.Loader import load_checker
    result = {}
    for key in originaldic:
        result[key] = load_checker(str(originaldic[key]))
    return result

class HostChannel(object):
    """A class that contains input and output string-named channels. Each channel must contain a Type object
    Any class which inherits from this must also inherit from HostChannel
    """
    def __init__(self, inputtypedict, outputtypedict):
        for key in inputtypedict:
            if not isinstance(key, str):
                raise TypeError
        for key in outputtypedict:
            if not isinstance(key, str):
                raise TypeError
        self.inputdefinition = inputtypedict
        self.outputdefinition = outputtypedict
        self.__inputchanneldic = _load_checker(inputtypedict)
        self.__outputchanneldic = _load_checker(outputtypedict)

    @property
    def inputchanneldic(self):
        return self.__inputchanneldic

    @property
    def outputchanneldic(self):
        return self.__outputchanneldic

class PythonTranslator(HostChannel):
    """ Python function based translator """
    def __init__(self, inputdic, outputdic, function):
        HostChannel.__init__(self, inputdic, outputdic)
        self._function = function

    def __call__(self, *args, **kwargs):
        for dickey in kwargs.keys():
            if not self.inputchanneldic[dickey].check(kwargs[dickey]):
                raise ValueError("Invalid value %s for input %s (%s)" % (kwargs[dickey], dickey, self))
        result = self._function(*args, **kwargs)
        return result

    def __str__(self):
        return "<PythonTranslator: %s, %s" % (self.inputdefinition, self.outputdefinition)

    @property
    def summary(self):
        inputdic = tuple(self.inputdefinition.values())
        outputdic = tuple(self.outputdefinition.values())
        result = {"iclass": "PythonTransformer", "input": inputdic, "output": outputdic}
        from pydsl.Abstract import ImmutableDict
        return ImmutableDict(result)

class PLYTranslator(object):
    def __init__(self, grammardefinition):
        self.module = grammardefinition.module

    def __call__(self, input):
        from ply import yacc, lex
        lexer = lex.lex(self.module)
        parser = yacc.yacc(debug=0, module = self.module)
        return {"output":parser.parse(input, lexer = lexer)}



#!/usr/bin/python
# -*- coding: utf-8 -*-
#This file is part of pydsl.
#
#pydsl is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.
#
#pydsl is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with pydsl.  If not, see <http://www.gnu.org/licenses/>.

"""Parser module"""

__author__ = "Nestor Arocha"
__copyright__ = "Copyright 2008-2013, Nestor Arocha"
__email__ = "nesaro@gmail.com"

import logging
LOG = logging.getLogger(__name__)


def terminal_symbol_reducer(symbol, word, production):
    """ Reduces a terminal symbol """
    from pydsl.Grammar.Tree import ParseTree
    if not isinstance(word, str):
        word = str(word)
    validresults = []
    if symbol.boundariesrules == "min":
        LOG.debug("terminal_symbol_reducer: policy: min")
        for begin in range(0, len(word)):
            for end in range(begin, len(word)+1):
                if symbol.check(word[begin:end]):
                    LOG.debug("terminal_symbol_reducer: parsed:"+ str(word[begin:end]))
                    validresults.append(ParseTree(begin, end, [symbol], word[begin:end], production))
                    break #found the smallest valid symbol at begin
    elif symbol.boundariesrules == "max":
        LOG.debug("terminal_symbol_reducer: policy: max")
        for begin in range(0, len(word)):
            maxword = 0
            for end in range(begin, len(word)+1):
                if symbol.check(word[begin:end]):
                    LOG.debug("terminal_symbol_reducer: parsed:"+ str(word[begin:end]))
                    maxword = end
            if maxword > 0:
                validresults.append(ParseTree(begin, maxword, [symbol], word[begin:maxword], production))
    elif symbol.boundariesrules == "any":
        LOG.debug("terminal_symbol_reducer: policy: any")
        for begin in range(0, len(word)):
            for end in range(begin, len(word)+1):
                if symbol.check(word[begin:end]):
                    validresults.append(ParseTree(begin, end, [symbol], word[begin:end], production))
    elif isinstance(symbol.boundariesrules , int):
        LOG.debug("terminal_symbol_reducer: policy: fixed")
        size = symbol.boundariesrules
        for begin in range(0, len(word)):
            if symbol.check(word[begin:begin + size]):
                LOG.debug("__auxReducer: parsed:"+ str(word[begin:begin + size]))
                validresults.append(ParseTree(begin, begin + size, [symbol], word[begin:begin + size], production))
    else:
        raise Exception("terminal_symbol_reducer: Unknown size policy")
    return validresults

def terminal_symbol_consume(symbol, word):
    """ Reduces a terminal symbol. Always start from left"""
    from pydsl.Grammar.Tree import ParseTree
    begin = 0
    if symbol.boundariesrules == "min":
        for end in range(begin, len(word)+1):
            if symbol.check(word[begin:end]):
                return [ParseTree(begin, end, [symbol], word[begin:end], symbol)]
    elif symbol.boundariesrules == "max":
        LOG.debug("terminal_symbol_consume: policy: max")
        maxword = 0
        for end in range(begin, len(word)+1):
            if symbol.check(word[begin:end]):
                LOG.debug("terminal_symbol_reducer: parsed:"+ str(word[begin:end]))
                maxword = end
        if maxword > 0:
           return[ParseTree(begin, maxword, [symbol], word[begin:maxword],
               symbol)]
    elif symbol.boundariesrules == "any":
        validresults = []
        for end in range(begin, len(word)+1):
            if symbol.check(word[begin:end]):
                validresults.append(ParseTree(begin, end, [symbol], word[begin:end], symbol))
        return validresults
    elif isinstance(symbol.boundariesrules,int):
        size = symbol.boundariesrules
        LOG.debug("terminal_symbol_consume: policy: fixed " + str(size))
        if len(word) >= size and symbol.check(word[:size]):
            return [ParseTree(0, size, [symbol], word[:size], symbol)]
    else:
        raise Exception("terminal_symbol_consume: Unknown size policy" + str(symbol.boundariesrules))
    return []

class Parser(object):
    """Parser abstract class. At this time, all parsers are tree based"""
    def __init__(self, bnfgrammar):
        self._productionset = bnfgrammar

    def get_trees(self, word): # -> list:
        """ returns a ParseTree list with all guesses """
        raise NotImplementedError

    def __call__(self, word):
        return self.get_trees(word)

    @property
    def productionset(self):
        """returns productionset"""
        return self._productionset

class TopDownParser(Parser):
    """Top down parser like descent parser"""
    def __init__(self, bnfgrammar):
        Parser.__init__(self, bnfgrammar)

class BottomUpParser(Parser):
    """ leaf to root parser"""
    def __init__(self, bnfgrammar):
        from pydsl.Memory.Loader import load_lexer
        self._lexer = load_lexer(bnfgrammar.alphabet())
        terminalsymbollist = bnfgrammar.terminalsymbollist
        Parser.__init__(self, bnfgrammar)
        



#!/usr/bin/python3

# -*- coding: utf-8 -*-
#This file is part of pydsl.
#
#pydsl is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.
#
#pydsl is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with pydsl.  If not, see <http://www.gnu.org/licenses/>.

"""
extracts input slices that are a Type 
"""

__author__ = "Nestor Arocha"
__copyright__ = "Copyright 2008-2013, Nestor Arocha"
__email__ = "nesaro@gmail.com"

import logging
from pydsl.Memory.Loader import load
from pydsl.Extract import extract

def extractaux(grammar, expression = None, input_file = None):
    #Generating and connecting output
    #listen to user, open read file, or other
    #configure output, write file, or other

    grammar = load(grammar)
    if expression:
        result = extract(grammar,expression)
        print(result)
        return result
    elif input_file:
        with open(input_file) as f:
            content = f.read()
        result = extract(grammar,content)
        print(result)
        return result
    raise Exception("No input file nor expression")

if __name__ == "__main__":
    import argparse
    TUSAGE = "usage: %(prog)s [options] type"
    PARSER = argparse.ArgumentParser(usage = TUSAGE)
    PARSER.add_argument("-d", "--debuglevel", action="store", type=int, dest="debuglevel", help="Sets debug level")
    PARSER.add_argument("-i", "--inputfile", action="store", dest="input_file", help="input filename dict")
    PARSER.add_argument("-e", "--expression", action="store", dest="expression", help="input expression")
    PARSER.add_argument("grammar", metavar="grammar", help="Grammar name")
    ARGS = vars(PARSER.parse_args())
    import sys
    DEBUGLEVEL = ARGS.pop('debuglevel') or logging.WARNING
    from pydsl.Config import load_default_memory
    load_default_memory()
    logging.basicConfig(level = DEBUGLEVEL)
    try:
        result = extractaux(**ARGS)
    except EOFError:
        sys.exit(0)
    if not result:
        sys.exit(-1)
    sys.exit(0)



#!/usr/bin/python
# -*- coding: utf-8 -*-
#This file is part of pydsl.
#
#pydsl is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.
#
#pydsl is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with pydsl.  If not, see <http://www.gnu.org/licenses/>.

"""loader class"""
from pydsl.Alphabet.Definition import Encoding

__author__ = "Nestor Arocha"
__copyright__ = "Copyright 2008-2013, Nestor Arocha"
__email__ = "nesaro@gmail.com"

def load_checker(grammar):
    from pydsl.Grammar.BNF import BNFGrammar
    from pydsl.Grammar.Definition import PLYGrammar, RegularExpressionDefinition, MongoGrammar, StringGrammarDefinition, PythonGrammar
    from pydsl.Alphabet.Definition import AlphabetDictDefinition
    if isinstance(grammar, str):
        grammar = load(grammar)
    if isinstance(grammar, BNFGrammar):
        from pydsl.Checker import BNFChecker
        return BNFChecker(grammar)
    elif isinstance(grammar, RegularExpressionDefinition):
        from pydsl.Checker import RegularExpressionChecker
        return RegularExpressionChecker(grammar)
    elif isinstance(grammar, PythonGrammar) or isinstance(grammar, dict) and "matchFun" in grammar:
        from pydsl.Checker import PythonChecker
        return PythonChecker(grammar)
    elif isinstance(grammar, MongoGrammar):
        from pydsl.Checker import MongoChecker
        return MongoChecker(grammar["spec"])
    elif isinstance(grammar, PLYGrammar):
        from pydsl.Checker import PLYChecker
        return PLYChecker(grammar)
    elif isinstance(grammar, AlphabetDictDefinition):
        from pydsl.Checker import AlphabetDictChecker
        return AlphabetDictChecker(grammar)
    elif isinstance(grammar, StringGrammarDefinition):
        from pydsl.Checker import StringChecker
        return StringChecker(grammar)
    elif isinstance(grammar, Encoding):
        from pydsl.Checker import EncodingChecker
        return EncodingChecker(grammar)
    else:
        raise ValueError(grammar)

def load_lexer(alphabet):
    from pydsl.Alphabet.Definition import AlphabetDictDefinition, AlphabetListDefinition
    if isinstance(alphabet, str):
        alphabet = load(alphabet)
    if isinstance(alphabet, AlphabetDictDefinition):
        from pydsl.Alphabet.Lexer import AlphabetDictLexer
        return AlphabetDictLexer(alphabet)
    if isinstance(alphabet, AlphabetListDefinition):
        from pydsl.Alphabet.Lexer import AlphabetListLexer
        return AlphabetListLexer(alphabet)
    elif isinstance(alphabet, Encoding):
        from pydsl.Alphabet.Lexer import EncodingLexer
        return EncodingLexer(alphabet)
    else:
        raise ValueError(alphabet)

def load_parser(grammar, parser = "auto"):
    if isinstance(grammar, str):
        grammar = load(grammar)
    from pydsl.Grammar.BNF import BNFGrammar
    if isinstance(grammar, BNFGrammar):
        if parser == "descent":
            from pydsl.Grammar.Parser.RecursiveDescent import RecursiveDescentParser
            return RecursiveDescentParser(grammar)
        elif parser in ("auto" , "default" , "weighted"):
            #TODO Guess best parser
            from pydsl.Grammar.Parser.Weighted import WeightedParser
            return WeightedParser(grammar)
        else:
            raise Exception("Wrong parser name: " + parser)
    else:
        raise ValueError(grammar)

def load_validator(grammar):
    if isinstance(grammar, str):
        grammar = load(grammar)
    from pydsl.Grammar.BNF import BNFGrammar
    if isinstance(grammar, BNFGrammar):
        from pydsl.Validate import BNFValidator
        return BNFValidator(grammar)
    else:
        raise ValueError(grammar)

def load_translator(function):
    if isinstance(function, str):
        function = load(function)
    from pydsl.Grammar.Definition import PLYGrammar
    if isinstance(function, PLYGrammar):
        from pydsl.Translator import PLYTranslator
        return PLYTranslator(function)
    if isinstance(function, dict):
        from pydsl.Translator import PythonTranslator
        return PythonTranslator(**function)
    raise ValueError(function)

def load(identifier, memorylist = None):
    if not memorylist:
        from pydsl.Config import GLOBALCONFIG
        memorylist = GLOBALCONFIG.memorylist
    results = search(identifier, memorylist)
    if not results:
        raise KeyError(identifier)
    if len(results) > 1:
        raise ValueError("Multiple results")
    identifier = list(results)[0]["identifier"]
    for memory in memorylist:
        if identifier in memory:
            return memory.load(identifier)
    raise KeyError(identifier)

def search(query, memorylist = None):
    if not memorylist:
        from pydsl.Config import GLOBALCONFIG
        memorylist = GLOBALCONFIG.memorylist
    from pydsl.Memory.Search.Searcher import MemorySearcher
    from pydsl.Memory.Search.Indexer import Indexer
    searcher = MemorySearcher([Indexer(x) for x in memorylist])
    return searcher.search(query)



#!/usr/bin/python
# -*- coding: utf-8 -*-
#This file is part of pydsl.
#
#pydsl is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.
#
#pydsl is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with pydsl.  If not, see <http://www.gnu.org/licenses/>.

"""Global (per execution) elements"""

__author__ = "Nestor Arocha"
__copyright__ = "Copyright 2008-2013, Nestor Arocha"
__email__ = "nesaro@gmail.com"

from pydsl.Abstract import Singleton
import logging
LOG = logging.getLogger(__name__)
from pkg_resources import resource_filename

def load_default_memory():
    from pydsl.Memory.Directory import DirStorage
    from pydsl.Memory.Dict import RegexpDictStorage
    from pydsl.Memory.List import EncodingStorage
    dirname = resource_filename("pydsl.contrib", "")
    GLOBALCONFIG.memorylist.append(DirStorage(dirname + "/grammar/"))
    GLOBALCONFIG.memorylist.append(RegexpDictStorage(dirname + "/dict/regexp.dict"))
    GLOBALCONFIG.memorylist.append(EncodingStorage(dirname + "/list/encoding.py"))
    GLOBALCONFIG.memorylist.append(DirStorage(dirname + "/transformer/"))

def default_formats():
    from pydsl.Memory.File.Regexp import load_re_from_file, summary_re_from_file
    from pydsl.Memory.File.BNF import load_bnf_file, summary_bnf_file
    from pydsl.Memory.File.Python import summary_python_file, load_python_file
    return [
        {"extension":".py",  "summary_from_file":summary_python_file, "load_from_file":load_python_file},
        {"extension":".re", "summary_from_file":summary_re_from_file,"load_from_file":load_re_from_file},
        {"extension":".bnf","summary_from_file":summary_bnf_file, "load_from_file":load_bnf_file},
        ]

class GlobalConfig(object):
    """Execution time global configuration"""
    def __init__(self, persistent_dir=None, debuglevel=40):
        self.persistent_dir = persistent_dir
        self.memorylist = []
        self.formatlist = default_formats()
        self.__debuglevel = debuglevel
        if self.persistent_dir is None:
            try:
                import os
                if not os.path.exists(os.environ['HOME'] + "/.pydsl/"):
                    os.mkdir(os.environ['HOME'] + "/.pydsl/")
                self.persistent_dir = os.environ['HOME'] + "/.pydsl/persistent/"
                if not os.path.exists(self.persistent_dir):
                    os.mkdir(self.persistent_dir)
            except (OSError, KeyError):
                LOG.exception("Unable to create persistent dir")

    def load(self, filename):
        """Load config from file"""
        raise NotImplementedError

    def save(self):
        """Save config to file"""
        raise NotImplementedError

    @property
    def debuglevel(self):
        return self.__debuglevel

    @debuglevel.setter
    def debuglevel(self, level):
        self.__debuglevel = level

GlobalConfig2 = Singleton('GlobalConfig2', (GlobalConfig, ), {})
GLOBALCONFIG = GlobalConfig2()  # The only instance available



#!/usr/bin/python

# -*- coding: utf-8 -*-
#This file is part of pydsl.
#
#pydsl is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.
#
#pydsl is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with pydsl.  If not, see <http://www.gnu.org/licenses/>.

"""
check if input data belongs to a Type
"""

__author__ = "Nestor Arocha"
__copyright__ = "Copyright 2008-2013, Nestor Arocha"
__email__ = "nesaro@gmail.com"

import logging
from pydsl.Memory.Loader import load_lexer

def bool_dict_values(dic):
    for key in dic:
        if str(dic[key]) == "False":
            dic[key] = False
        else:
            dic[key] = bool(dic[key])
    return dic

def lexer(alphabet, expression = None, inputfile = None):
    #Generating and connecting output
    #listen to user, open read file, or other
    #configure output, write file, or other
    lexer = load_lexer(alphabet)
    if expression:
        result = [str(x) for x in lexer(expression)]
        print(result)
    elif inputfile:
        with open(inputfile, 'rb') as f:
            expression = f.read()
        result = [str(x) for x in lexer(expression)]
        print(result)
    else:
        raise Exception
    return True

if __name__ == "__main__":
    import argparse
    TUSAGE = "usage: %(prog)s [options] alphabet"
    PARSER = argparse.ArgumentParser(usage = TUSAGE)
    PARSER.add_argument("-d", "--debuglevel", action="store", type=int, dest="debuglevel", help="Sets debug level")
    PARSER.add_argument("-i", "--inputfile", action="store", dest="inputfile", help="input filename")
    PARSER.add_argument("-e", "--expression", action="store", dest="expression", help="input expression")
    PARSER.add_argument("alphabet", metavar="alphabet", help="Alphabet name")
    ARGS = PARSER.parse_args()
    if not ARGS.expression and not ARGS.inputfile:
        PARSER.error("expression or inputfile required")
    ARGS = vars(ARGS)
    import sys
    DEBUGLEVEL = ARGS.pop("debuglevel") or logging.WARNING
    logging.basicConfig(level = DEBUGLEVEL)
    from pydsl.Config import load_default_memory
    load_default_memory()
    try:
        result = lexer(**ARGS)
    except EOFError:
        sys.exit(0)
    if not result:
        sys.exit(-1)
    sys.exit(0)



#!/usr/bin/python
# -*- coding: utf-8 -*-
#This file is part of pydsl.
#
#pydsl is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.
#
#pydsl is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with pydsl.  If not, see <http://www.gnu.org/licenses/>.

""" Directory storage """

from .Memory import Memory
from pydsl.Memory.File.Python import getFileTuple
from pydsl.Abstract import ImmutableDict
from pydsl.Config import GLOBALCONFIG
import logging
LOG = logging.getLogger(__name__)


class DirStorage(Memory):
    """A collection of elements stored inside a directory"""
    def __init__(self, dirpath):
        Memory.__init__(self)
        self.path = dirpath
        from pydsl.Memory.Search.Searcher import MemorySearcher
        self._searcher = MemorySearcher(self)

    def __iter__(self):
        self.index = 0
        self.cache = []
        for filename in self.all_files():
            try:
                self.cache.append(self.summary_from_filename(filename))
            except (AttributeError,ImportError, TypeError) as e:
                LOG.debug("Error while loading %s file summary %s" % (filename, e) )
        return self

    @property
    def allowed_extensions(self):
        return [x["extension"] for x in GLOBALCONFIG.formatlist]

    def next(self):
        try:
            result = self.cache[self.index]
        except IndexError:
            raise StopIteration
        self.index += 1
        return result
        

    @staticmethod
    def summary_from_filename(filepath):
        entry = [x for x in GLOBALCONFIG.formatlist if filepath.endswith(x["extension"])][0]
        return ImmutableDict(entry["summary_from_file"](filepath))

    def all_files(self):
        import glob
        extensions = self.allowed_extensions or [""]
        for extension in extensions:
            searchstring = self.path + "*" + extension
            tmpresult = glob.glob(searchstring)
            for result in tmpresult:
                if result.endswith("__init__.py"):
                    continue
                yield result


    def all_names(self):
        """Generates all Static Ids"""
        for fullname in self.all_files():
            (_, _, fileBaseName, fileExtension) = getFileTuple(fullname)
            if self.allowed_extensions and fileExtension not in self.allowed_extensions:
                continue
            yield fileBaseName.split(".")[0]

    def load(self, name):
        result = self._searcher.search(name)
        if len(result) > 1:
            LOG.error("Found two or more matches, FIXME: processing the first, should raise exception")
        if not result:
            raise KeyError(self.__class__.__name__ + name)
        filepath = list(result)[0]["filepath"]
        entries = [x for x in GLOBALCONFIG.formatlist if filepath.endswith(x["extension"])]
        for entry in entries:
            try:
                return entry["load_from_file"](filepath)
            except ValueError:
                continue
        raise ValueError("Unable to open: %s" % name)


    def __contains__(self, key):
        return key in self.all_names()

    def provided_iclasses(self):
        return set([x['iclass'] for x in self])




#!/usr/bin/python
# -*- coding: utf-8 -*-
#This file is part of pydsl.
#
#pydsl is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.
#
#pydsl is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with pydsl.  If not, see <http://www.gnu.org/licenses/>.

"""Base Lexer classes"""

__author__ = "Nestor Arocha"
__copyright__ = "Copyright 2008-2013, Nestor Arocha"
__email__ = "nesaro@gmail.com"

import logging
LOG = logging.getLogger(__name__)
from pydsl.Memory.Loader import load_checker
from pydsl.Alphabet.Token import Token
unknownchar = "UNKNOWN"

class AlphabetTranslator(object):
    @property
    def input_alphabet(self):
        raise NotImplementedError

    @property
    def output_alphabet(self):
        raise NotImplementedError

class EncodingLexer(AlphabetTranslator):
    """Special Lexer that encodes from a string a reads a string"""
    def __init__(self, encoding):
        self.encoding = encoding

    def __call__(self, string):
        for x in string:
            yield Token(x)

class Lexer(AlphabetTranslator):
    """Lexer follows an alphabet definition.
    generates a list of tokens and it
    is always described with a regular grammar"""
    def __init__(self, generate_unknown=False):
        self.load(None)
        self.generate_unknown = generate_unknown

    @property
    def current(self):
        """Returns the element under the cursor"""
        try:
            return self.string[self.index]
        except IndexError:
            return None

    def load(self, string):
        self.string = string
        self.index = 0

    def consume(self):
        self.index += 1

    def match(self, char):
        if self.current == char:
            self.consume()
        else:
            raise Exception("%s doesn't match %s"%(self.current,char))

    def nextToken(self):
        raise NotImplementedError

    def __call__(self, string):# -> "TokenList":
        """Tokenizes input, generating a list of tokens"""
        self.string = string
        while True:
            result = [x for x in self.nextToken()]
            return result


class AlphabetDictLexer(Lexer):
    def __init__(self, alphabet):
        Lexer.__init__(self)
        self.alphabet = alphabet

    @property
    def current(self):
        """Returns the element under the cursor until the end of the string"""
        return self.string[self.index:]

    def nextToken(self):
        while self.current:
            validelements = []
            for x,y in self.alphabet.grammardict.items():
                for first_element in y.first:
                    from pydsl.Grammar.Definition import GrammarDefinition
                    if not isinstance(first_element, GrammarDefinition):
                        print(first_element.__class__.__name__, first_element, x)
                        raise Exception
                    checker = load_checker(first_element)
                    if checker.check(self.current[0]):
                        validelements.append((x,y))
                        break
            if not validelements:
                if not self.generate_unknown:
                    raise Exception("Not found")
                string = self.current[0]
                self.consume()
                yield Token(unknownchar, string)
            elif len(validelements) == 1:
                element = validelements[0][1]
                checker = load_checker(element)
                for size in range(element.maxsize or len(self.current), element.minsize, -1):
                    if checker.check(self.current[:size]):
                        break
                else:
                    raise Exception("Nothing consumed")
                string = self.current[:size]
                for _ in range(size):
                    self.consume()
                yield Token(string, validelements[0][1])
            else:
                raise Exception("Multiple choices")

class AlphabetListLexer(Lexer):
    def __init__(self, alphabet):
        Lexer.__init__(self)
        self.alphabet = alphabet

    @property
    def current(self):
        """Returns the element under the cursor until the end of the string"""
        return self.string[self.index:]

    def nextToken(self):
        while self.current:
            from pydsl.Grammar.Definition import  StringGrammarDefinition
            currentgd = StringGrammarDefinition(self.current[0])
            validelements = [x for x in self.alphabet.grammar_list if currentgd in x.first]
            if not validelements:
                if not self.generate_unknown:
                    raise Exception("Not found")
                string = self.current[0]
                self.consume()
                yield Token(unknownchar, string)
            elif len(validelements) == 1:
                element = validelements[0]
                checker = load_checker(element)
                for size in range(element.maxsize or len(self.current), max(element.minsize-1,0), -1):
                    if checker.check(self.current[:size]):
                        break
                else:
                    raise Exception("Nothing consumed")
                string = self.current[:size]
                for _ in range(size):
                    self.consume()
                yield Token(string, validelements[0])
            else:
                raise Exception("Multiple choices" + str([str(x) for x in validelements]))
